// ==============================================================
// Ananke vessel class - a LEO momentum exchange tether
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#ifndef ANANKE_H
#define ANANKE_H

#define STRICT
#define ORBITER_MODULE
#define _CRT_SECURE_NO_DEPRECATE

#include "math.h"
#include "orbitersdk.h"
#include "ZTCMaths.h"
#include "AnankeMeshManager.h"
#include "AnankeTypeDefs.h"
#include <vector>


// ==============================================================
// Type definitions
// ==============================================================


// ==============================================================
// Class definition
// ==============================================================

class Ananke: public VESSEL2 {
public:
	Ananke(OBJHANDLE hVessel, int flightmodel)
		: VESSEL2(hVessel, flightmodel), meshMgr(hVessel) {}

	// VESSEL2 overloaded callback functions
	void clbkSetClassCaps(FILEHANDLE cfg);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	void clbkPreStep(double SimT, double SimDT, double mjd);
	void clbkPostCreation();
	void clbkVisualCreated(VISHANDLE vis, int refcount);
	void clbkVisualDestroyed(VISHANDLE vis, int refcount);
	bool clbkLoadGenericCockpit();
	bool clbkLoadVC(int id);

	// custom functions
	double DLLEXPORT getPayloadVVAngle();
	bool DLLEXPORT hasPayload();
	bool DLLEXPORT releasePayload();

private:
	static const double facilityFuel;	// fuel tank size
	LightState beaconState; // cache's the status of the lights
	MeshManager meshMgr; // manages meshes and beacons
	UINT catcherDockIndex;	// For AnankeControl interface

};


// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel);

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel);

#endif // ANANKE_H