// ==============================================================
// Ananke Beacon Manager class - manages the beacons for the
//                               Ananke class vessel
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#include "AnankeBeaconManager.h"

// --------------------------------------------------------------
// Constructor
// --------------------------------------------------------------
BeaconManager::BeaconManager() {}

// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
BeaconManager::~BeaconManager()
{
	UINT i;

	for (i=0;i<beacons.size();i++)
	{
		delete beacons.at(i)->bs;
		delete beacons.at(i);
	}
}

// --------------------------------------------------------------
// Initialise the beacons
// --------------------------------------------------------------
void BeaconManager::init(OBJHANDLE hV, MeshId mI, const VECTOR3 &mP)
{
	UINT i;
	
	// cache mesh/vessel properties
	hVessel = hV;
	meshId = mI;
	meshPos = mP;
	v = (VESSEL2*)oapiGetVesselInterface(hVessel);

	switch (meshId)
	{
		case CATCHER:
			beacons.reserve(6);

			for (i=0;i<6;i++)
			{
				beacons.push_back(new Beacon);
				beacons.at(i)->bs = new BEACONLIGHTSPEC;
				
				switch (i)
				{
					case 0:
						beacons.at(i)->pos=_V(0, 3.02, -73.27)+meshPos;
						beacons.at(i)->beaconType=FORE;
						break;
					case 1:
						beacons.at(i)->pos=_V(0, -3.02, -73.27)+meshPos;
						beacons.at(i)->beaconType=AFT;
						break;
					case 2:
						beacons.at(i)->pos=_V(-3.02, 0, -73.27)+meshPos;
						beacons.at(i)->beaconType=PORT;
						break;
					case 3:
						beacons.at(i)->pos=_V(3.02, 0, -73.27)+meshPos;
						beacons.at(i)->beaconType=STBD;
						break;
					case 4:
						beacons.at(i)->pos=_V(-3.87, 0, -26.35)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 5:
						beacons.at(i)->pos=_V(3.87,  0, -26.35)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
				}
			}
			break;
		case CABLE_A:
			beacons.reserve(4);

			for (i=0;i<4;i++)
			{
				beacons.push_back(new Beacon);
				beacons.at(i)->bs = new BEACONLIGHTSPEC;

				switch (i)
				{
					case 0:
						beacons.at(i)->pos=_V(4.6, 0, -20000.42)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 1:
						beacons.at(i)->pos=_V(-4.6, 0, -20000.42)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 2:
						beacons.at(i)->pos=_V(-4.6, 0, 1.57)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 3:
						beacons.at(i)->pos=_V(4.6, 0, 1.57)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
				}
			}
			break;
		case CABLE_B:
			beacons.reserve(4);

			for (i=0;i<4;i++)
			{
				beacons.push_back(new Beacon);
				beacons.at(i)->bs = new BEACONLIGHTSPEC;

				switch (i)
				{
					case 0:
						beacons.at(i)->pos=_V(4.6, 0, -20000.42)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 1:
						beacons.at(i)->pos=_V(-4.6, 0, -20000.42)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 2:
						beacons.at(i)->pos=_V(-4.6, 0, 1.57)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 3:
						beacons.at(i)->pos=_V(4.6, 0, 1.57)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
				}
			}
			break;
		case FACILITY:
			beacons.reserve(8);

			for (i=0;i<8;i++)
			{
				beacons.push_back(new Beacon);
				beacons.at(i)->bs = new BEACONLIGHTSPEC;

				switch (i)
				{
					case 0:
						beacons.at(i)->pos=_V(-.37, .35, 20036.37)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 1:
						beacons.at(i)->pos=_V(.45, -.20, 20018.33)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 2:
						beacons.at(i)->pos=_V(-2.33, 0.48, 19996.69)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 3:
						beacons.at(i)->pos=_V(-.52, 2.12, 19993.58)+meshPos;
						beacons.at(i)->beaconType=PORT;
						break;
					case 4:
						beacons.at(i)->pos=_V(2.46, .38, 19993.6)+meshPos;
						beacons.at(i)->beaconType=STBD;
						break;
					case 5:
						beacons.at(i)->pos=_V(.45, .4, -20036.37)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 6:
						beacons.at(i)->pos=_V(.45, .4, -20018.33)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
					case 7:
						beacons.at(i)->pos=_V(.91, -1.89, -19993.87)+meshPos;
						beacons.at(i)->beaconType=CABLE;
						break;
				}
			}
			break;
	}

	for (i=0;i<beacons.size();i++)
	{
		beacons.at(i)->bs->falloff = 0;
		beacons.at(i)->bs->active = false;

		switch (beacons.at(i)->beaconType)
		{
			case FORE:
				beacons.at(i)->color=_V(1,1,1); //white
				beacons.at(i)->trueSize = 0.7;
				beacons.at(i)->bs->size = beacons.at(i)->trueSize;
				beacons.at(i)->bs->shape = BEACONSHAPE_STAR;
				beacons.at(i)->bs->period = 0;
				beacons.at(i)->falloff = 0.6;
				break;
			case AFT:
				beacons.at(i)->color=_V(1,1,1); //white
				beacons.at(i)->trueSize = 0.7;
				beacons.at(i)->bs->size = beacons.at(i)->trueSize;
				beacons.at(i)->bs->shape = BEACONSHAPE_STAR;
				beacons.at(i)->bs->period = 0;
				beacons.at(i)->falloff = 0.3;
				break;
			case PORT:
				beacons.at(i)->color=_V(1,0,0); //red
				beacons.at(i)->trueSize = 0.7;
				beacons.at(i)->bs->size = beacons.at(i)->trueSize;
				beacons.at(i)->bs->shape = BEACONSHAPE_STAR;
				beacons.at(i)->bs->period = 0;
				beacons.at(i)->falloff = 0.3;
				break;
			case STBD:
				beacons.at(i)->color=_V(0,1,0); //green
				beacons.at(i)->trueSize = 0.7;
				beacons.at(i)->bs->size = beacons.at(i)->trueSize;
				beacons.at(i)->bs->shape = BEACONSHAPE_STAR;
				beacons.at(i)->bs->period = 0;
				beacons.at(i)->falloff = 0.3;
				break;
			case CABLE:
				beacons.at(i)->color=_V(1,1,1); //white
				beacons.at(i)->trueSize = 0.7;
				beacons.at(i)->bs->size = beacons.at(i)->trueSize;
				beacons.at(i)->bs->shape = BEACONSHAPE_STAR;
				beacons.at(i)->bs->period = 0;
				beacons.at(i)->bs->duration = 0.25;
				beacons.at(i)->bs->tofs = 0;
				beacons.at(i)->falloff = 0.7;
				break;
		}
		beacons.at(i)->bs->pos = &(beacons.at(i)->pos);
		beacons.at(i)->bs->col = &(beacons.at(i)->color);
		v->AddBeacon(beacons.at(i)->bs);
	}
}

// --------------------------------------------------------------
// Shift the beacons
// --------------------------------------------------------------
void BeaconManager::shiftBeacons(VECTOR3 &shiftVector)
{
	UINT i;

	for (i=0;i<beacons.size();i++)
		beacons.at(i)->pos = beacons.at(i)->pos - shiftVector;
}

// --------------------------------------------------------------
// Scale the beacons - a custom scale function is required
//                     because the built in scaling is suitable
//                     only for vessels up to 50m in size
// --------------------------------------------------------------
void BeaconManager::scaleBeacons()
{
	VECTOR3 gCamPos; //global camera position
	VECTOR3 gBeaconPos; //global beacon position
	VECTOR3 r;
	double camDist; //distance from beacon to camera
	double scaleFactor; //beacon size scale factor
	UINT i;
	
	// Scale beacons
	oapiCameraGlobalPos(&gCamPos);
	for (i=0;i<beacons.size();i++)
	{
		v->Local2Global(beacons.at(i)->pos,gBeaconPos);
		r = gBeaconPos - gCamPos;
		camDist = length(r);
		if (camDist > 50)
			scaleFactor = pow((camDist/50),beacons.at(i)->falloff);
		else
			scaleFactor = 1;
		beacons.at(i)->bs->size = beacons.at(i)->trueSize * scaleFactor;
	}
}

// --------------------------------------------------------------
// Turn beacons on/off
// --------------------------------------------------------------
void BeaconManager::switchBeacons(LightState ls)
{
	UINT i;

	for (i=0;i<beacons.size();i++)
	{
		if (ls)
			beacons.at(i)->bs->active = true;
		else
			beacons.at(i)->bs->active = false;
	}
}
