// ==============================================================
// Ananke Beacon Manager class - manages the beacons for the
//                               Ananke class vessel
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#ifndef ANANKE_BEACON_MANAGER_H
#define ANANKE_BEACON_MANAGER_H

#define STRICT
#define _CRT_SECURE_NO_DEPRECATE

#include "math.h"
#include "orbitersdk.h"
#include "ZTCMaths.h"
#include "AnankeTypeDefs.h"
#include <vector>

// ==============================================================
// Type definitions
// ==============================================================

// --------------------------------------------------------------
// BeaconType - identifies the beacon type
// --------------------------------------------------------------
enum BeaconType {FORE,AFT,PORT,STBD,CABLE};

// --------------------------------------------------------------
// Beacon class - holds a single beacon
// --------------------------------------------------------------
class Beacon {
public:
	VECTOR3 color, pos;
	double trueSize; //true beacon size
	double falloff; //rate of change of beacon size with distance
	BEACONLIGHTSPEC *bs;
	BeaconType beaconType;
};

// --------------------------------------------------------------
// BeaconManager class - manages all beacons
// --------------------------------------------------------------
class BeaconManager {
public:
	BeaconManager();
	~BeaconManager();
	void init(OBJHANDLE hV, MeshId mI, const VECTOR3 &mP);
	void shiftBeacons(VECTOR3 &shiftVector);
	void scaleBeacons();
	void switchBeacons(LightState ls);
private:
	OBJHANDLE hVessel;
	MESHHANDLE hMesh;
	VESSEL2 *v;
	MeshId meshId;
	VECTOR3 meshPos;
	std::vector<Beacon*> beacons;
};

#endif // ANANKE_BEACON_MANAGER_H