// ==============================================================
// Ananke Light Globe Manager class - manages the light globes
//                                    mesh materials for the
//                                    Ananke class vessel
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#include "AnankeLightGlobeManager.h"

// --------------------------------------------------------------
// Constructor
// --------------------------------------------------------------
LightGlobeManager::LightGlobeManager() {}

// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
LightGlobeManager::~LightGlobeManager()
{
	UINT i;

	for (i=0;i<lightGlobes.size();i++)
	{
		delete lightGlobes.at(i);
	}
}

// --------------------------------------------------------------
// Initialise the light globes
// --------------------------------------------------------------
void LightGlobeManager::init(OBJHANDLE hV, MESHHANDLE hM, MeshId mI)
{
	UINT i;
	
	// cache mesh/vessel properties
	hVessel = hV;
	hMesh = hM;
	meshId = mI;

	switch (meshId)
	{
		case CATCHER:
			lightGlobes.reserve(3);

			for (i=0;i<3;i++)
			{
				lightGlobes.push_back(new LightGlobe);

				switch ( (LightGlobeType)i )
				{
					case WHITE:
						lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,16);
						lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
						lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
						lightGlobes.at(i)->onState.specular = _C(1,1,1,1);
						lightGlobes.at(i)->onState.ambient = _C(1,1,1,1);
						lightGlobes.at(i)->onState.diffuse = _C(1,1,1,1);
						lightGlobes.at(i)->onState.emissive = _C(1,1,1,1);
						break;
					case RED:
						lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,17);
						lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
						lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
						lightGlobes.at(i)->onState.specular = _C(1,0,0,1);
						lightGlobes.at(i)->onState.ambient = _C(1,0,0,1);
						lightGlobes.at(i)->onState.diffuse = _C(1,0,0,1);
						lightGlobes.at(i)->onState.emissive = _C(1,0,0,1);
						break;
					case GREEN:
						lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,18);
						lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
						lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
						lightGlobes.at(i)->onState.specular = _C(0,1,0,1);
						lightGlobes.at(i)->onState.ambient = _C(0,1,0,1);
						lightGlobes.at(i)->onState.diffuse = _C(0,1,0,1);
						lightGlobes.at(i)->onState.emissive = _C(0,1,0,1);
						break;
				}
			}
			
			break;
		case CABLE_A:
			lightGlobes.reserve(1);

			lightGlobes.push_back(new LightGlobe);

			i=0;
			lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,7);
			lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
			lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
			lightGlobes.at(i)->onState.specular = _C(1,1,1,1);
			lightGlobes.at(i)->onState.ambient = _C(1,1,1,1);
			lightGlobes.at(i)->onState.diffuse = _C(1,1,1,1);
			lightGlobes.at(i)->onState.emissive = _C(1,1,1,1);
			
			break;
		case CABLE_B:
			lightGlobes.reserve(1);

			lightGlobes.push_back(new LightGlobe);

			i=0;
			lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,7);
			lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
			lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
			lightGlobes.at(i)->onState.specular = _C(1,1,1,1);
			lightGlobes.at(i)->onState.ambient = _C(1,1,1,1);
			lightGlobes.at(i)->onState.diffuse = _C(1,1,1,1);
			lightGlobes.at(i)->onState.emissive = _C(1,1,1,1);
			
			break;
		case FACILITY:
			lightGlobes.reserve(3);

			for (i=0;i<3;i++)
			{
				lightGlobes.push_back(new LightGlobe);

				switch ( (LightGlobeType)i )
				{
					case WHITE:
						lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,17); //TODO: replace xxx with material index
						lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
						lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
						lightGlobes.at(i)->onState.specular = _C(1,1,1,1);
						lightGlobes.at(i)->onState.ambient = _C(1,1,1,1);
						lightGlobes.at(i)->onState.diffuse = _C(1,1,1,1);
						lightGlobes.at(i)->onState.emissive = _C(1,1,1,1);
						break;
					case RED:
						lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,18); //TODO: replace xxx with material index
						lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
						lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
						lightGlobes.at(i)->onState.specular = _C(1,0,0,1);
						lightGlobes.at(i)->onState.ambient = _C(1,0,0,1);
						lightGlobes.at(i)->onState.diffuse = _C(1,0,0,1);
						lightGlobes.at(i)->onState.emissive = _C(1,0,0,1);
						break;
					case GREEN:
						lightGlobes.at(i)->hMaterial = oapiMeshMaterial(hMesh,19); //TODO: replace xxx with material index
						lightGlobes.at(i)->offState = *(lightGlobes.at(i)->hMaterial);
						lightGlobes.at(i)->onState = lightGlobes.at(i)->offState;
						lightGlobes.at(i)->onState.specular = _C(0,1,0,1);
						lightGlobes.at(i)->onState.ambient = _C(0,1,0,1);
						lightGlobes.at(i)->onState.diffuse = _C(0,1,0,1);
						lightGlobes.at(i)->onState.emissive = _C(0,1,0,1);
						break;
				}
			}
			
			break;
	}
}

// --------------------------------------------------------------
// Purge light globes when the visual is destroyed
// --------------------------------------------------------------
void LightGlobeManager::purge()
{
	UINT i;

	for (i=0;i<lightGlobes.size();i++)
	{
		delete lightGlobes.at(i); // de-allocate LightGlobe objects
	}

	lightGlobes.resize(0); // delete LightGlobe pointers ready for use next time
}

// --------------------------------------------------------------
// Turn light globes on/off
// --------------------------------------------------------------
void LightGlobeManager::switchLightGlobes(LightState ls)
{
	UINT i;

	for (i=0;i<lightGlobes.size();i++)
	{
		if (ls)
			*(lightGlobes.at(i)->hMaterial) = lightGlobes.at(i)->onState;
		else
			*(lightGlobes.at(i)->hMaterial) = lightGlobes.at(i)->offState;
	}
}