// ==============================================================
// Ananke Light Globe Manager class - manages the light globes
//                                    mesh materials for the
//                                    Ananke class vessel
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#ifndef ANANKE_LIGHT_GLOBE_MANAGER_H
#define ANANKE_LIGHT_GLOBE_MANAGER_H

#define STRICT
#define _CRT_SECURE_NO_DEPRECATE

#include "orbitersdk.h"
#include "AnankeTypeDefs.h"
#include "ZTCMaths.h"
#include <vector>

// ==============================================================
// Type definitions
// ==============================================================

// --------------------------------------------------------------
// LightGlobeType - identifies the light globe type
// --------------------------------------------------------------
enum LightGlobeType {WHITE,RED,GREEN};

// --------------------------------------------------------------
// LightGlobe class - holds a reference to single light globe
//                    material
// --------------------------------------------------------------
class LightGlobe {
public:
	MATERIAL *hMaterial;
	MATERIAL offState;
	MATERIAL onState;
};

// --------------------------------------------------------------
// LightGlobeManager class - manages all light globes
// --------------------------------------------------------------
class LightGlobeManager {
public:
	LightGlobeManager();
	~LightGlobeManager();
	void init(OBJHANDLE hV, MESHHANDLE hM, MeshId mI);
	void switchLightGlobes(LightState ls);
	void purge();
private:
	OBJHANDLE hVessel;
	MESHHANDLE hMesh;
	MeshId meshId;
	std::vector<LightGlobe*> lightGlobes;
};

#endif // ANANKE_LIGHT_GLOBE_MANAGER_H