// ==============================================================
// Ananke Mesh Manager class - manages the meshes for the
//                             Ananke class vessel
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#include "AnankeMeshManager.h"

// --------------------------------------------------------------
// Constructor
// --------------------------------------------------------------
MeshManager::MeshManager(OBJHANDLE hV)
		   :hVessel(hV) {}

// --------------------------------------------------------------
// Destructor
// --------------------------------------------------------------
MeshManager::~MeshManager()
{
	UINT i,j;

	for (i=0;i<meshes.size();i++)
	{
		for (j=0;j < meshes.at(i)->masses.size();j++)
			delete meshes.at(i)->masses.at(j);
		delete meshes.at(i);
	}
}

// --------------------------------------------------------------
// Member initialization
// --------------------------------------------------------------
const double MeshManager::cableMassRate = 0.125;

// --------------------------------------------------------------
// Initialise the meshes
// --------------------------------------------------------------
void MeshManager::init()
{
	UINT i;
	double size=0;

	v = (VESSEL2*)oapiGetVesselInterface(hVessel);

	// load in the mesh/mass information
	meshes.reserve(numberOfMeshes);
	for (i=0;i<numberOfMeshes;i++)
	{
		meshes.push_back(new Mesh);
		switch ((MeshId)i)
		{
			case CATCHER:
				meshes.at(i)->fileName = "ztc-catcher";
				meshes.at(i)->pos = _V(0.0,0.0,-100093.027);
				meshes.at(i)->size = 73.65;

				meshes.at(i)->masses.push_back(new MassObject);
				meshes.at(i)->masses.at(0)->massDistribution = PNT; // Just a point for the catcher. We will ignore the cables for simplicity
				meshes.at(i)->masses.at(0)->mass = 2600;
				meshes.at(i)->masses.at(0)->pos = _V(0,0,0);
				meshes.at(i)->masses.at(0)->radius = _V(0,0,0);
				break;

			case CABLE_A:
				meshes.at(i)->fileName = "ztc-CableA";
				meshes.at(i)->pos = _V(0.0,0.0,-40016.227);
				meshes.at(i)->size = 20001.05;

				meshes.at(i)->masses.push_back(new MassObject); // cable 1
				meshes.at(i)->masses.at(0)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(0)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(0)->pos = _V(3.5,0,0);
				meshes.at(i)->masses.at(0)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 2
				meshes.at(i)->masses.at(1)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(1)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(1)->pos = _V(-3.5,0,0);
				meshes.at(i)->masses.at(1)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 3
				meshes.at(i)->masses.at(2)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(2)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(2)->pos = _V(0,3.5,0);
				meshes.at(i)->masses.at(2)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 4
				meshes.at(i)->masses.at(3)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(3)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(3)->pos = _V(0,-3.5,0);
				meshes.at(i)->masses.at(3)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);
				break;

			case CABLE_B:
				meshes.at(i)->fileName = "ztc-CableA";
				meshes.at(i)->pos = _V(0.0,0.0,-80018.327);
				meshes.at(i)->size = 20001.05;

				meshes.at(i)->masses.push_back(new MassObject); // cable 1
				meshes.at(i)->masses.at(0)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(0)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(0)->pos = _V(3.5,0,0);
				meshes.at(i)->masses.at(0)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 2
				meshes.at(i)->masses.at(1)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(1)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(1)->pos = _V(-3.5,0,0);
				meshes.at(i)->masses.at(1)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 3
				meshes.at(i)->masses.at(2)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(2)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(2)->pos = _V(0,3.5,0);
				meshes.at(i)->masses.at(2)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 4
				meshes.at(i)->masses.at(3)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(3)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(3)->pos = _V(0,-3.5,0);
				meshes.at(i)->masses.at(3)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);
				break;

			case FACILITY:
				meshes.at(i)->fileName = "ztc-Ananke-massesA";
				meshes.at(i)->pos = _V(0,0,0);
				meshes.at(i)->size = 20036.37;

				meshes.at(i)->masses.push_back(new MassObject); // Climber
				meshes.at(i)->masses.at(0)->massDistribution = PNT;
				meshes.at(i)->masses.at(0)->mass = 106904;
				meshes.at(i)->masses.at(0)->pos = _V(0,0,-20000); //TODO: check this number against actual mesh position
				meshes.at(i)->masses.at(0)->radius = _V(0,0,0);

				meshes.at(i)->masses.push_back(new MassObject); // Counterweight
				meshes.at(i)->masses.at(1)->massDistribution = PNT;
				meshes.at(i)->masses.at(1)->mass = 106904;
				meshes.at(i)->masses.at(1)->pos = _V(0,0,20000); //TODO: check this number against actual mesh position
				meshes.at(i)->masses.at(1)->radius = _V(0,0,0);

				meshes.at(i)->masses.push_back(new MassObject); // cable 1
				meshes.at(i)->masses.at(2)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(2)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(2)->pos = _V(3.5,0,0);
				meshes.at(i)->masses.at(2)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 2
				meshes.at(i)->masses.at(3)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(3)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(3)->pos = _V(-3.5,0,0);
				meshes.at(i)->masses.at(3)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 3
				meshes.at(i)->masses.at(4)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(4)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(4)->pos = _V(0,3.5,0);
				meshes.at(i)->masses.at(4)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);

				meshes.at(i)->masses.push_back(new MassObject); // cable 4
				meshes.at(i)->masses.at(5)->massDistribution = ROD; // Just distributed mass for the cables. Reels ignored for simplicity
				meshes.at(i)->masses.at(5)->mass = meshes.at(i)->size * 2 * cableMassRate;
				meshes.at(i)->masses.at(5)->pos = _V(0,-3.5,0);
				meshes.at(i)->masses.at(5)->radius = meshes.at(i)->masses.at(0)->pos + _V(0,0,meshes.at(i)->size);
				break;
		}
		meshes.at(i)->beaconMgr.init(hVessel,(MeshId)i,meshes.at(i)->pos); // initialise BeaconManager
		size += meshes.at(i)->size;
	}

	calcCog();
	shiftMeshes(cog,false);
	for (i=0;i<meshes.size();i++) {
		meshes.at(i)->meshIdx = v->AddMesh(meshes.at(i)->fileName.c_str(),&(meshes.at(i)->pos)); // add mesh to vessel
		v->SetMeshVisibilityMode(meshes.at(i)->meshIdx,MESHVIS_ALWAYS); // make it visible
	}

	calcPmi();

	v->SetSize(size);
	v->SetClipRadius(length(meshes.at((UINT)CATCHER)->pos) + meshes.at((UINT)CATCHER)->size);
	v->SetVisibilityLimit(0.05);
}

// --------------------------------------------------------------
// Tasks required at each time step
// --------------------------------------------------------------
void MeshManager::preStep()
{
	UINT i;

	// scale beacons
	for (i=0;i<meshes.size();i++)
	{
		meshes.at(i)->beaconMgr.scaleBeacons();
	}
}

// --------------------------------------------------------------
// Tasks required after visual created
// --------------------------------------------------------------
void MeshManager::visualCreated(VISHANDLE vis)
{
	UINT i;

	for (i=0;i<meshes.size();i++)
	{
		meshes.at(i)->beaconMgr.scaleBeacons();
		meshes.at(i)->beaconMgr.switchBeacons(ON);
		meshes.at(i)->lightGlobeMgr.init(hVessel,v->GetMesh(vis,i),(MeshId)i);
		meshes.at(i)->lightGlobeMgr.switchLightGlobes(ON);
	}
}

// --------------------------------------------------------------
// Tasks required after visual destroyed
// --------------------------------------------------------------
void MeshManager::visualDestroyed(VISHANDLE vis)
{
	UINT i;

	for (i=0;i<meshes.size();i++)
	{
		meshes.at(i)->beaconMgr.switchBeacons(OFF);
		meshes.at(i)->lightGlobeMgr.switchLightGlobes(OFF);
		meshes.at(i)->lightGlobeMgr.purge();
	}
}

// --------------------------------------------------------------
// Calculate centre of gravity
// --------------------------------------------------------------
void MeshManager::calcCog()
{
	UINT i,j;
	VECTOR3 massRadiusProduct;
	double totalMass;

	massRadiusProduct = _V(0,0,0);
	totalMass = 0;

	// add the masses associated with the meshes
	for (i=0;i<meshes.size();i++)
	{
		for (j=0;j<meshes.at(i)->masses.size();j++)
		{
			massRadiusProduct = massRadiusProduct + (meshes.at(i)->pos + meshes.at(i)->masses.at(j)->pos) * meshes.at(i)->masses.at(j)->mass;
			totalMass += meshes.at(i)->masses.at(j)->mass;
		}
	}

	v->SetEmptyMass(totalMass);

	// add the fuel mass. Assume it is located at the origin of the facility mesh
	massRadiusProduct = massRadiusProduct + meshes.at((UINT)FACILITY)->pos * v->GetFuelMass();
	totalMass += v->GetFuelMass();
	
	// calculate the vector from current CoG to new CoG
	cog = massRadiusProduct/totalMass;
}

// --------------------------------------------------------------
// Calculate PMIs
// --------------------------------------------------------------
void MeshManager::calcPmi()
{
	UINT i,j,k;
	VECTOR3 pmi, vBuf;
	double len, pmiBuf; // length in-plane;

	pmi = _V(0,0,0);

	for (i=0;i<meshes.size();i++)
	{
		for (j=0;j<meshes.at(i)->masses.size();j++)
		{
			pmiBuf = 0;

			switch (meshes.at(i)->masses.at(j)->massDistribution)
			{
				case ROD:
					for (k=0;k<3;k++) 
					{
						vBuf = meshes.at(i)->masses.at(j)->radius - meshes.at(i)->masses.at(j)->pos; // get vector from CoG to end
						vBuf.data[k] = 0; // project vector into plane perpendicular to axis of rotation
						len = 2 * length( vBuf ); // calculate length in-plane
						pmiBuf = meshes.at(i)->masses.at(j)->mass * pow(len,2) / 12; // calculate the pmi about the mass CoG
						vBuf = meshes.at(i)->masses.at(j)->pos + meshes.at(i)->pos; // position of mass from vessel CoG
						vBuf.data[k] = 0; // project vector into plane perpendicular to axis of rotation
						pmi.data[k] += pmiBuf + meshes.at(i)->masses.at(j)->mass * pow(length(vBuf),2); // re-calc PMI about vessel CoG
					}
					break;
				case PNT:
					for (k=0;k<3;k++)
					{
						vBuf = meshes.at(i)->masses.at(j)->pos + meshes.at(i)->pos; // position of mass from vessel CoG
						vBuf.data[k] = 0; // project vector into plane perpendicular to axis of rotation
						pmi.data[k] += meshes.at(i)->masses.at(j)->mass * pow(length(vBuf),2); // calculate PMI
					}
					break;
				//TODO: add cylinder
			}
		}
	}

	//TODO: add fuel tanks

	v->SetPMI(pmi);
}

// --------------------------------------------------------------
// Shift the meshes and optionally the centre of gravity
// --------------------------------------------------------------
void MeshManager::shiftMeshes(VECTOR3 &shiftVector, bool shiftCG)
{
	UINT i,j;

	for (i=0;i<meshes.size();i++)
	{
		meshes.at(i)->beaconMgr.shiftBeacons(shiftVector);
		meshes.at(i)->pos = meshes.at(i)->pos - shiftVector;
		for (j=0;j<meshes.at(i)->masses.size();j++)
			meshes.at(i)->masses.at(j)->pos = meshes.at(i)->masses.at(j)->pos - shiftVector;
	}

	if (shiftCG)
		v->ShiftCG(shiftVector);
}

// --------------------------------------------------------------
// Switch the beacons on/off
// --------------------------------------------------------------
void MeshManager::switchBeacons(LightState ls)
{
	UINT i;

	for (i=0;i<meshes.size();i++)
	{
		meshes.at(i)->beaconMgr.switchBeacons(ls);
		meshes.at(i)->lightGlobeMgr.switchLightGlobes(ls);
	}
}

// --------------------------------------------------------------
// Load virtual cockpits
// --------------------------------------------------------------
bool MeshManager::loadVC(Vc_Pos vcPos)
{
	VECTOR3 dockPos, dockDir, dockRot, camPos;

	const int NONE = -1;

	switch (vcPos)
	{
	case PAYLOAD_VIEW_NEAR:
		camPos = _V(0,-18.273,-27.225) + meshes.at((UINT)CATCHER)->pos;
		oapiVCSetNeighbours(NONE,NONE,NONE,(int)PAYLOAD_VIEW_FAR);
		break;
	case PAYLOAD_VIEW_FAR:
		camPos = _V(0,18.012,-27.219) + meshes.at((UINT)CATCHER)->pos;
		oapiVCSetNeighbours(NONE,NONE,(int)PAYLOAD_VIEW_NEAR,NONE);
		break;
	}

	v->SetCameraOffset(camPos);
	
	v->GetDockParams(v->GetDockHandle(CATCHER_DOCK_INDEX), dockPos, dockDir, dockRot);
	v->SetCameraDefaultDirection(Normalise(dockPos - camPos),PI);

	v->SetCameraRotationRange(PI,PI,PI/2,PI/2);

	return true;
}
