// ==============================================================
// Ananke Payload Release Dialog - gives automatic payload
// release for the Ananke LEO tether
//
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#include "AnankePayloadRelease.h"


// ==============================================================
// API interface
// ==============================================================

// --------------------------------------------------------------
// This function is called when Orbiter starts or when the module
// is activated.
// --------------------------------------------------------------
DLLCLBK void InitModule (HINSTANCE hDLL)
{
	g_hInst = hDLL; // remember the instance handle

	// To allow the user to open our new dialog box, we create
	// an entry in the "Custom Functions" list which is accessed
	// in Orbiter via Ctrl-F4.
	g_dwCmd = oapiRegisterCustomCmd ("Ananke Payload Release",
		"Opens the Ananke Payload Release dialog",
		OpenDlgClbk, NULL);
}

// --------------------------------------------------------------
// This function is called when Orbiter shuts down or when the
// module is deactivated
// --------------------------------------------------------------
DLLCLBK void ExitModule (HINSTANCE hDLL)
{
	// Unregister the custom function in Orbiter
	oapiUnregisterCustomCmd (g_dwCmd);
}

// --------------------------------------------------------------
// This function is called before each time step
// --------------------------------------------------------------
DLLCLBK void opcPreStep (double SimT,double SimDT,double mjd)
{
	HWND hDlg;
	double VVAngle, dVVAngle;
//	bool HasPayload, PayloadReleased;
	char OutText[10];

	if (hDlg = oapiFindDialog(g_hInst,DLG_ANANKE_PAYLOAD_RELEASE))
	{
/*		if (!(g_Ananke->clbkConsumeBufferedKey(COMM_HasPayload, false, (char *)&HasPayload)))
		{
			sprintf(oapiDebugString(),"KeyComm error getting payload status");
			return;
		}
		*/
		if ( g_Ananke->hasPayload() )
		{
			// Get velocity vector angle
/*			if (!(g_Ananke->clbkConsumeBufferedKey(COMM_GetPayloadVVAngle, false, (char *)&VVAngle)))
			{
				sprintf(oapiDebugString(),"KeyComm error getting velocity vector angle");
				return;
			}
			*/
			VVAngle = g_Ananke->getPayloadVVAngle();
			
			// Update rate of change
			dVVAngle = VVAngle - g_oldVVAngle;
			g_oldVVAngle = VVAngle;

			// If likely to go past 0deg in next time step, then release payload
			if (g_ReleaseArmed && ((VVAngle+dVVAngle)<0) && (VVAngle>0))
			{
/*				if (!(g_Ananke->clbkConsumeBufferedKey(COMM_ReleasePayload, false, (char *)&PayloadReleased)))
				{
					sprintf(oapiDebugString(),"KeyComm error releasing paylod");
					return;
				} */
				if ( g_Ananke->releasePayload() )
				{
					g_ReleaseArmed = false;
					SetWindowText (GetDlgItem (hDlg, CMD_ARM), "Arm Payload Release");
				}
				else
				{
					sprintf(oapiDebugString(),"Ananke error releasing paylod");
					return;
				}
			}

			// Update timer and update window contents if it is time
			if (dlgUpdateTime >= dlgUpdateInterval)
			{
				dlgUpdateTime = 0.0;
				VVAngle = Rad2Deg(VVAngle);
				sprintf(OutText,"%.2f",VVAngle);
				SetWindowText (GetDlgItem (hDlg, LBL_VVANGLE), OutText);
			}
			else
				dlgUpdateTime += SimDT;
		}
		else
		{
			SetWindowText (GetDlgItem (hDlg, CMD_ARM), "Arm Payload Release");
			SetWindowText (GetDlgItem (hDlg, LBL_VVANGLE), "-xxx.xx");
		}
	}
}

// --------------------------------------------------------------
// Open the dialog window
// --------------------------------------------------------------
void OpenDlgClbk (void *context)
{
	DWORD cnt;
	BOOL ANANKE_FOUND=FALSE;
	VESSEL *v;
	char OutText[25];

	// Save the window handle
	HWND hDlg = oapiOpenDialogEx (g_hInst, DLG_ANANKE_PAYLOAD_RELEASE, MsgProc, DLG_CAPTIONCLOSE);

	// Search for the Ananke vessel - only single vessel presently supported
	for (cnt=0; cnt<oapiGetVesselCount(); cnt++)
	{
		g_obj_Ananke = oapiGetVesselByIndex(cnt);
		v = oapiGetVesselInterface(g_obj_Ananke);
		if (v->GetClassName() != NULL)
			if (ANANKE_FOUND = !strcmp(v->GetClassName(),"Ananke"))
				break;
	}

	// Check to see if we found an Ananke class vessel
	if (!ANANKE_FOUND)
	{
		sprintf(oapiDebugString(), "Ananke-class vessel not found!");
		CloseDlg(hDlg);
		return;
	}

	g_Ananke = (Ananke *)v; // Since we know it is Ananke, it is safe to implicit cast to Ananke

	sprintf(OutText,"Tether: %s",g_Ananke->GetName());
	SetWindowText (GetDlgItem (hDlg, GRP_VESSEL_NAME), OutText);

	dlgUpdateTime = dlgUpdateInterval; // Force display update
}

// --------------------------------------------------------------
// Close the dialog
// --------------------------------------------------------------
void CloseDlg (HWND hDlg)
{
	oapiCloseDialog (hDlg);
}

// --------------------------------------------------------------
// Windows message handler for the dialog box
// --------------------------------------------------------------
BOOL CALLBACK MsgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD (wParam)) {

		case CMD_ARM:  // user pressed dialog button
			if (g_ReleaseArmed)
			{
				g_ReleaseArmed = false;
				SetWindowText (GetDlgItem (hDlg, CMD_ARM), "Arm Payload Release");
				return TRUE;
			}

/*			if (!(g_Ananke->clbkConsumeBufferedKey(COMM_HasPayload, false, (char *)&g_ReleaseArmed)))
			{
				sprintf(oapiDebugString(),"KeyComm error getting payload status");
				return TRUE;
			}
			*/			
			if ( g_Ananke->hasPayload() )
			{
				g_ReleaseArmed = true;
				SetWindowText (GetDlgItem (hDlg, CMD_ARM), "Disarm Payload Release");
			return TRUE;
			}
		
		case IDCANCEL:
			g_ReleaseArmed = false;
			CloseDlg(hDlg);
			return TRUE;
		}
		break;
	}
	return oapiDefDialogProc (hDlg, uMsg, wParam, lParam);
}
