// ==============================================================
// ZTC_Phantom vessel class - a small vessel use for mission
// planning
// 
// Copyright 2008 - Tim Blaxland and Sean Kilpatrick
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"


// ==============================================================
// Class interface
// ==============================================================

class ZTC_Phantom: public VESSEL2 {
public:
	ZTC_Phantom (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel) {}
	void clbkSetClassCaps (FILEHANDLE cfg);
};

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void ZTC_Phantom::clbkSetClassCaps (FILEHANDLE cfg)
{
	// physical specs
	SetSize (0.5);
	SetEmptyMass (1000.0);
	SetPMI (_V(1.0,1.0,1.0));
	SetCameraOffset (_V(0,0,0));

	// propellant resources
	PROPELLANT_HANDLE hpr = CreatePropellantResource (500.0);

	// thrusters
	THRUSTER_HANDLE th_main = CreateThruster (_V(0,0,-4.35), _V(0,0,1), 1e8, hpr, 6e3);
	CreateThrusterGroup (&th_main, 1, THGROUP_MAIN);
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ZTC_Phantom (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ZTC_Phantom*)vessel;
}
